/*//////////////////////////////////////////////////////////////////////////////

Copyright Laurent Excoffier, Genetics and Biometry Lab, University of Geneva
E-mail: laurent.excoffier@anthro.unige.ch
URL: http://anthropologie.unige.ch/~laurent

The use of the following source code in only allowed if it is kept intact
with its original banner

//////////////////////////////////////////////////////////////////////////////*/



#include "genealgy.h"
#include "public.h"
#include "time.h"
#include "migrmat.h"
#include "deme.h"

#include "cstring.h"
#include <fstream.h>

#include "coalmain.h"
#include "cond_var.h"

//------------------------------------------------------------------------------
int
TMeanMat::compute_mean(const int& n) {
	_num_updates=n   ;
	if (!_num_updates) return 0;
	for (int i=0; i<_size; ++i) {
   	for (int j=0; j<_size; ++j) {
      	_data[i][j]/=_num_updates;
      }
   }
   return 1;
};

//Compute the s.d. from the values present in the matrix, assuming that
//those are the sum of square values
int 
TMeanMat::compute_sd(const int& n, const TMeanMat& mean) {
   if (n<2) return 0;
	_num_updates=n   ;
	if (!_num_updates) return 0;
	for (int i=0; i<_size; ++i) {
   	for (int j=0; j<_size; ++j) {
      	double s2=_data[i][j],
         		 m=mean._data[i][j]; //assumes that mean contains mean values
      	_data[i][j]=sqrt( (s2-n*m*m)/(n-1) );
      }
   }
   return 1;
};

int
TMeanMat::update_with(const TMigrationMatrix& MM) {
	if (!MM.size() || _size!=MM.size()) return 0;
   for (int i=0; i<_size; ++i) {
   	for (int j=0; j<_size; ++j) {
      	_data[i][j]+=MM(i,j);
      }
   }return 1;
};

int
TMeanMat::update_with_square(const TMigrationMatrix& MM) {
	if (!MM.size() || _size!=MM.size()) return 0;
   for (int i=0; i<_size; ++i) {
   	for (int j=0; j<_size; ++j) {
      	_data[i][j]+=MM(i,j)*MM(i,j);
      }
   }return 1;
};
//------------------------------------------------------------------------------
void my_strrev(char * str) {
   int size=strlen(str);
   char *buf = new char[size+1];
   for (int i=0; i<size; ++i) {
      buf[i]=str[size-1-i];
   }
   for (int i=0; i<size; ++i) {
      str[i]=buf[i];
   }
   delete[] buf;
}
//------------------------------------------------------------------------------
my_string remove_extension(const my_string& s) {
  	my_string sout;
   char sin1[200], sin2[200];
   strcpy(sin1,s.c_str());
   //my_strrev(sin1);
   int i, len = strlen(sin1);
   //First: invert my_string;
   my_strrev(sin1);
   for (i=0; i<len ; ) {
      sin2[i]=sin1[i];
   	if (sin1[i]=='.') break;
      ++i;
   }
   sin2[i+1]='\0';
   my_strrev(sin2);
   my_strrev(sin1);
   char* c=strstr(sin1,sin2);
   if (!c) return "";
   c[0]='\0';
   sout=sin1;
   return sout;
}
//------------------------------------------------------------------------------
my_string extract_path(const my_string& s) {
	my_string sout;
   char sin1[400], sin2[400];
   strcpy(sin1,s.c_str());
   int i, len=strlen(sin1);
   //First: invert my_string;
   my_strrev(sin1);
   for (i=0; i<len ; ) {
      sin2[i]=sin1[i];  
   	if (sin1[i]=='\\') break;
      ++i;
   }
   sin2[i+1]='\0';
   my_strrev(sin2);
   my_strrev(sin1);
   if (strlen(sin1)==strlen(sin2)) {   //it means that no backslash was detected
   	sout="";
      return sout;
   }
   char* c=strstr(sin1,sin2);
   if (!c) return "";
   c[1]='\0';

   sout=sin1;
   return sout;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

typedef MY_TArrayAsVector<my_string> NameArray;

float UNIT_TIME;

int main(int argc, char *argv[]) {

	my_string usage;

   usage ="\nsim_coale.exe usage:\n";
   usage+="\n [1] : sim_coal <no arguments>";
   usage+="\n        The program will prompt you for an input file name to process and the";
   usage+="\n        number of simulations to perform";
   usage+="\n [2] : sim_coal <batch file name>";
   usage+="\n        The program will open the batch file that should contain, on the first";
   usage+="\n        line, the number of simulations to perform, and on the following lines";
   usage+="\n        the names of the input files to process";
   usage+="\n [3] : sim_coal <file name> <number of simulations to perform>\n";

   NameArray infile_name(1,5);

   my_string batchFile;
   ifstream ifs_batch;

   my_string Path="";

   int num_rand_samples=1;
   int num_files=0;

   if (argc==2) {
   	//Then one supposes that one receives a file containing the name of
      //the files to be processed
      batchFile=argv[1];
      Path=extract_path(batchFile);
      ifs_batch.open(batchFile.c_str());
      if (!ifs_batch) {
      	cout << "Unable to open input file (" << batchFile << ")\n";
         cout << usage;
			return -1;
      }
      else {           
         	my_string line;
      	//Number of random samples to draw must be on first line
      	ifs_batch >> num_rand_samples;
         line.read_to_delim(ifs_batch);
      	while (ifs_batch) {
         	line.read_to_delim(ifs_batch);
            if (!line.is_null()) {
      			++num_files;
               infile_name.Add(Path+remove_extension(line));
            }
      	}
         ifs_batch.close();
      }
   }
   else
   if (argc==3) {
   	//Then one supposes that one receives a file name to be processed
      //and the number of permutations to be performed
      num_files=1;
      batchFile=argv[1];
      Path=extract_path(batchFile);
      ifs_batch.open(batchFile.c_str());
      if (!ifs_batch) {
      	cout << "Unable to open input file (1) (" << batchFile << ")" << endl;
         cout << usage;
			return -1;
      }
      else {
      	num_rand_samples=atoi(argv[2]);
         ifs_batch.close();
         infile_name.Add(Path+remove_extension(batchFile));
         if (!num_rand_samples) cout << usage;
      }
   }
   else {
      num_files=1;
      my_string curName;
      cout << "\n\nGeneric input file name  (*.par)  : ";
      cin >> curName;
      infile_name.Add(curName);
      cout << "\n\nNo. of random samples to generate : ";
      cin >> num_rand_samples;
   }

	   //Reads and generate genealogies for the required number of files

   for (int nf=0; nf<num_files; ++nf) {

   	//Creating input and output file names
   	my_string	outfile_name,
      				arl_batch_file_name,
               	arl_generic_name,
      				paup_batch_file_name,   
               	paup_generic_name,
               	paup_name, log_file_name,
               	paup_true_trees_name, paup_mut_trees_name;

		outfile_name			= infile_name[nf]+".gen";
      arl_batch_file_name 	= infile_name[nf]+".arb";
      arl_generic_name		= infile_name[nf];
      paup_generic_name		= infile_name[nf];
      paup_batch_file_name = infile_name[nf]+".bat";
      paup_name				= infile_name[nf]+".paup";
      paup_true_trees_name	= infile_name[nf]+"_true_trees.trees";
      paup_mut_trees_name	= infile_name[nf]+"_mut_trees.trees";
      log_file_name 			= infile_name[nf]+".log";
   	infile_name[nf]+=".par";

   	//Opening result files
   	ofstream ofs(outfile_name.c_str());

   #ifdef _PRINT_ARLEQUIN_OUTPUT_
      ofstream ArlBatch(arl_batch_file_name.c_str());
   #endif

   #ifdef _PRINT_SEQ_PAUP_BATCH_
      ofstream PaupBatch(paup_batch_file_name.c_str());
   #endif

      //cout << "\nOutput file name : " << outfile_name << "\n";
   	if (!ofs) {
   		cout 	<< "\nUnable to open output file (" << outfile_name << ")\n";
      	continue;
   	}

   	//Initialization of a bunch of object and vectors
   	TDemeCollection 	myDemes;
 		TEventArray 		myEvents(10,10);  
 		TDemeSizeArray 	myDemeSizes(10,10);
 		TGrowthArray 		myGrowthRates(10,10);
		TSampSizeArray 	mySampleSizes(10,10);
      TMigrMatArray     myMigrationMatrices(1,10);  //Loro_16_9_99

   	//Initialize input file
   	ifstream ifs;
      ifs.open(infile_name[nf].c_str());

   	if (!ifs) {
      	cout << "Unable to open input file (2)(" << infile_name[nf] << ")" << endl;
			continue;
   	}

   	int num_pop;
   	my_string line;


   	line.read_to_delim(ifs);   //Reads a blank line

   	ifs >> num_pop;        //Reads the number of populations to simulate
		line.read_to_delim(ifs);

   	//Read deme sizes
   	line.read_to_delim(ifs);   //Reads a blank line
   		long cur_size;
   	for (int i=0; i<num_pop; ++i) {
      	ifs >> cur_size;
      	line.read_to_delim(ifs);
   		myDemeSizes.Add(cur_size);
   	}
   
   	cout << "\nDeme sizes\n";
   	for (int i=0; i<num_pop; ++i) {
   		cout << "Deme " << i << "\t" << myDemeSizes[i] << "\n";
   	}
   	cout << "\n";


   	//Reads the sample sizes
   	line.read_line(ifs);  //Reads a blank line
   	for (int i=0; i<num_pop; ++i) {
      	ifs >> cur_size;
      	line.read_to_delim(ifs);
   		mySampleSizes.Add(cur_size);
   	}

		int tot_num_nodes=0;
   	cout << "\nSample sizes\n";
   	for (int i=0; i<num_pop; ++i) {
      	tot_num_nodes+=mySampleSizes[i];
   		cout << "Deme " << i << "\t" << mySampleSizes[i] << "\n";
   	}
   	cout << endl;

   	line.read_line(ifs);  //Reads a blank line
   		float cur_growth;
   	for (int i=0; i<num_pop; ++i) {
   		ifs >> cur_growth;
      	line.read_to_delim(ifs);
   		myGrowthRates.Add(cur_growth);
   	}

   	cout << "Growth rates\n";
   	for (int i=0; i<num_pop; ++i) {
   		cout << i << "\t" << myGrowthRates[i] << "\n";
   	}
   	cout << endl;

      //Loro_16_9_99
      //Reading how many migration matrices to read
      int numMigMat=0; //Default
   	line.read_line(ifs);  //Reads a blank line
      ifs >> numMigMat;
      line.read_line(ifs);

      //Loro_16_9_99
      if (numMigMat)
      for (int m=0; m<numMigMat; ++m) {
   		//Reading migration matrix
   		TMigrationMatrix myMigrationRates(num_pop);
   		line.read_line(ifs);  //Reads a blank line
   		for (int i=0; i<num_pop; ++i) {
   			for (int j=0; j<num_pop; ++j) {
      			ifs >> myMigrationRates(i,j);
            }
      	}
         myMigrationMatrices.Add(myMigrationRates);
         //	  TMigrationMatrix test_mat(2, STEPPING_STONE_3D, 0.01);
   		cout << "\nMigration matrix\n";
   		cout << myMigrationMatrices[m];
      	line.read_to_delim(ifs); //Reads until the end of line
   	}
      else { //Then a model without migration is assumed
      	TMigrationMatrix myMigrationRates(num_pop); //Enough to create an ^matrix filled with zeroes   
         myMigrationMatrices.Add(myMigrationRates);
      }

   	int num_events;
   	//Read historical events
  		line.read_line(ifs); //Reads a blank line
   	ifs >> num_events;   //Reads the number of historical events to read
   	line.read_line(ifs);
   	for (int i=0; i<num_events; ++i) {
   		THistoricalEvent curevent;
   		ifs >> curevent;
   		myEvents.Add(curevent);
   	}
   	cout << "\nHistorical events\n";
   	for (int i=0; i<num_events; ++i) {
   		cout << "Event " << i << "\n" << myEvents[i] << "\n";
   	}
   	cout << endl;
      
      //Loro_26_2_99 to avoid a bug when no historical event is defined
      if (num_events) line.read_to_delim(ifs); //Reads until the end of line

   	//Read mutation rate
  		float mut_rate;
      line.read_line(ifs); //Reads a blank line
      ifs >> mut_rate;
      cout << "\nMutation rate per generation : " << mut_rate;
      line.read_to_delim(ifs); //Reads until the end of line

      //Read number of loci
      int num_loci;
      line.read_line(ifs);
      ifs >> num_loci;
      cout << "\nNumber of loci to simulate : " << num_loci;
      line.read_to_delim(ifs); //Reads until the end of line

      //Reads data type
      Mut_Type data_type;
      float transition_rate=0.0;
      //Loro_26_2_99
      int range_constraint=0; //A value of zero assumes no range constraint
      line.read_line(ifs);  //Read comment line
      line.read_to_delim(ifs, ' ');
      line.to_upper();
      //Loro_26_2_99
      if (line.contains("MICROSAT")) {
      	data_type=MICROSAT;
         ifs >> range_constraint; //Extract range constraint for microsat data
      	line.read_to_delim(ifs);
      }
      else
      if (line.contains("DNA")) {
      	data_type=DNA;
         ifs >> transition_rate; //Extract transition rate for DNA
      	line.read_to_delim(ifs);
      }
      else
      if (line.contains("RFLP")) data_type=RFLP;

      // Loro_28_2_99 : Be careful: only one node allowed to be created before
      // building the tree

		TNode first_node;
      first_node.reset_node_count(); //Do not forget to reset the node count...

      if (data_type==MICROSAT && range_constraint) {//Fix minimum and maximum size for microsats
			double y=2.0, x=range_constraint;
         if (fmod(x,y)==0.0) { //Even number
         	first_node.min_mic=-(range_constraint/2)+1;
            first_node.max_mic=range_constraint/2;
         }
         else {
         	first_node.min_mic=(-range_constraint+1)/2;
            first_node.max_mic=(range_constraint-1)/2;
         }
      }

      cout << "\nData type : ";
      switch (data_type) {
      	case DNA: 	cout 	<< "DNA\n" << "Transition rate : " << transition_rate
         						<< "\n";
         			 	break;
         case RFLP:  cout 	<< "RFLP\n"; break;
         case MICROSAT : if (range_constraint)
         						cout
         							<< "MICROSAT"
                           	<< "\nRange constraint : " << range_constraint
                           	<< " (" << (10000+first_node.min_mic) << ";"
                           	<< (10000+first_node.max_mic) << ")\n";
         					 else
                         	cout
         							<< "MICROSAT"
                           	<< "\n No range constraint\n";
                         break;
         default : break;
      }
      
      //Loro_26_2_99
      double gamma_a=0.0, mut_ratio=1.0, prop_sites=1.0, num_rate_categories=0;
   	if (data_type==DNA) {
      	//Reads gamma parameter
      	line.read_line(ifs);
      	ifs >> gamma_a;
      	cout << "\nGamma parameter : " << gamma_a;
      	//If gamma parameter is negative, then it means that we have a
      	//two mutation rate model. So read the mutation ratio and the proportion
      	//of sites with the high mutation rate
      	if (gamma_a<0.0) ifs >> mut_ratio >> prop_sites;
      	else
      	if (gamma_a>0.0) ifs >> num_rate_categories;
      }

      //loro_27_2_99
      ifs.close();

   	//Initialization of the demes
   	myDemes.create_demes(&myMigrationMatrices);
		myDemes.initialize_deme_size(myDemeSizes);
   	myDemes.initialize_growth_rates(myGrowthRates);
   	myDemes.initialize_events(myEvents);
   	myDemes.create_lineages(mySampleSizes);

   	//Output initial conditions
   	ofs 	<< "\nDemes initial conditions"
         	<< "\n========================\n"
         	<< myDemes
         	<< endl;

   	//Initialize the generator of random numbers once for all
   	time_t start=clock();
   	long lidum=-start; ran3(&lidum);  lidum=1;
   	cout << "Random generator initialized with : " << start << endl;
      ofs  << "Random generator initialized with : " << start << "\n\n";

   	cout << "\nBuilding " << num_rand_samples << " genealogies ...\n";
	#ifdef _COMPUTE_MOMENTS_
   	//A matrix to store mean coalescent times within and among populations
   	TMeanMat MeanCoalMat(myDemes.num_demes(), 0.0);
   	//A matrix to store s.d. coalescent times within and among populations
   	TMeanMat sdCoalMat(myDemes.num_demes(), 0.0);

      //A matrix to store mean number of mutations within and among populations
   	TMeanMat MeanPairDiff(myDemes.num_demes(), 0.0);
   	//A matrix to store s.d. number of mutations within and among populations
   	TMeanMat sdPairDiff(myDemes.num_demes(), 0.0);
   #endif

	#ifdef _PRINT_SEQ_PAUP_
		//Open PAUP* output file
      ofstream
      	PaupFile(paup_name.c_str());
   #endif

   #ifdef _PRINT_TREE_PAUP_
   	ofstream
         PaupTrueTreesFile(paup_true_trees_name.c_str()),
         PaupMutTreesFile(paup_mut_trees_name.c_str());
   #endif

   #ifdef _PRINT_SEQ_PAUP_
	   PaupFile
         << "#NEXUS\n"
      	<< "\n[Simulated data generated by program sim_coal.exe (Laurent Excoffier)"
         << "\n\tSimulation conditions:"
         << "\n\tNo. of taxa   : " << tot_num_nodes
         << "\n\tNo. of loci   : " << num_loci
         << "\n\tNo. of demes  : " << num_pop
         << "\n\tMutation rate : " << (mut_rate/num_loci) << " per locus (site) per generation"
         << "\n\tAlpha         : " << gamma_a << " (0=homogeneity)"
         << "\n\t%Transitions  : " << (100*transition_rate) << " (ts/tv=";
      if (transition_rate!=1)
      	PaupFile << (transition_rate/(1-transition_rate));
      else
			PaupFile << "inf";
      PaupFile << ")\n]\n\n";  
      write_PAUP_header(PaupFile,log_file_name.c_str());
  	#endif

   #ifdef _PRINT_TREE_PAUP_
      write_PAUP_trees_section_header(PaupTrueTreesFile);
      write_PAUP_trees_section_header(PaupMutTreesFile);
  	#endif


      float mut_rate_per_locus=0.0;
      if (num_loci) mut_rate_per_locus=mut_rate/num_loci;
      Mut_Model mut_mod;

      if (fabs(gamma_a)>1e-7) mut_mod=K80_GAMMA;
      else mut_mod=K80_NOGAMMA;
      
      tree_output_type tree_type;   

      //Build unequal mutation rates if needed
      //Create the vector of gamma distributed mutation rates once for all

   	//Loro_26_2_99
   	if (!gamma_a==0.0 && data_type==DNA) {
      	cout << "\nInitializing unequal mutation rates...";
   		first_node.mut_rates.get_new_rates(num_loci,gamma_a, mut_ratio,prop_sites,
         											num_rate_categories);
      	cout << "done\n\n";
   	}

     	int tot_cases=0;
      my_float tree_mut_length=0.0, tree_mut_length2=0.0,
      			tree_exp_length=0.0, tree_exp_length2=0.0;

   	for ( ; tot_cases<num_rand_samples; ++tot_cases) {

   		if (myDemes.build_tree()) {

         	//Add mutations
      		long mut=myDemes.sprinkle_mutations(mut_rate, num_loci,
						data_type, gamma_a, mut_ratio, prop_sites, transition_rate,
                  range_constraint);
            tree_mut_length+=mut;
            tree_mut_length2+=mut*mut;
            tree_exp_length+=first_node.tree_exp_length();
            tree_exp_length2+=first_node.tree_exp_length()*first_node.tree_exp_length();

         #ifdef _PRINT_SEQ_PAUP_
				//Writing data to PAUP file
            write_PAUP_replicate_number((tot_cases+1), PaupFile);
            write_PAUP_data_header(tot_num_nodes, num_loci, data_type,PaupFile);
            myDemes.write_samples_to_PAUP_file(PaupFile, data_type);
				write_PAUP_end_matrix(PaupFile);
				write_PAUP_tree_header(PaupFile, (tot_cases+1));
            tree_type=MUT_RATE;
            myDemes.print_gene_tree(PaupFile, tree_type, mut_rate_per_locus);
            write_PAUP_end(PaupFile);
            if (num_loci) {
            	PaupFile
            		<< "\n\t[Tree length = " << (first_node.tree_exp_length()/num_loci)
                  << "\n\tNumber of sites hit by mutations = "  << first_node.count_polym_sites()
                  << "]\n";
            }
				write_PAUP_block(PaupFile,mut_mod);
         #endif

         #ifdef _PRINT_TREE_PAUP_
			   write_PAUP_tree_name(PaupTrueTreesFile, (tot_cases+1));
            write_PAUP_tree_name(PaupMutTreesFile, (tot_cases+1));
            tree_type=MUT_RATE;
            myDemes.print_gene_tree(PaupTrueTreesFile, tree_type, mut_rate_per_locus);
            tree_type=NUM_MUT;
            myDemes.print_gene_tree(PaupMutTreesFile, tree_type, mut_rate_per_locus);
         #endif

            if (!fmod((tot_cases+1),10))
         		cout << "\nGenealogy # " << (tot_cases+1) << "/" << num_rand_samples;

         
   		#ifdef _PRINT_SEQ_PAUP_BATCH_
				//Build paup file name               
            my_string PaupOut, PaupLog;
  				int dec, sign, ndig = 0;
            PaupOut = paup_generic_name + "_" + fcvt(tot_cases,ndig, &dec, &sign) + ".pau";
            PaupLog = paup_generic_name + "_" + fcvt(tot_cases,ndig, &dec, &sign) + ".log";
            //Open paup output file
      		ofstream PaupOutFile(PaupOut.c_str());
            
            //Write name in paup batch file
            PaupBatch << "paup " << PaupOut << "\n";
            PaupOutFile
         		<< "#NEXUS\n"
      			<< "\n[Simulated data generated by program sim_coal.exe (Laurent Excoffier)"
         		<< "\n\tSimulation conditions:"
         		<< "\n\tNo. of taxa   : " << tot_num_nodes
         		<< "\n\tNo. of loci   : " << num_loci
         		<< "\n\tNo. of demes  : " << num_pop
         		<< "\n\tMutation rate : " << (mut_rate/num_loci) << " per locus (site) per generation"
         		<< "\n\tAlpha         : " << gamma_a << " (0=homogeneity)"
         		<< "\n\t%Transitions  : " << (100*transition_rate) << " (ts/tv=";
            if (transition_rate!=1) PaupOutFile << (transition_rate/(1-transition_rate));
      		else	PaupOutFile << "inf";
      		PaupOutFile << ")\n]\n\n";

            write_PAUP_header(PaupOutFile,PaupLog.c_str());
            write_PAUP_replicate_number((tot_cases+1), PaupOutFile);
            write_PAUP_data_header(tot_num_nodes, num_loci, data_type,PaupOutFile);
            myDemes.write_samples_to_PAUP_file(PaupOutFile, data_type);
				write_PAUP_end_matrix(PaupOutFile);
				write_PAUP_tree_header(PaupOutFile, (tot_cases+1));
            tree_type=MUT_RATE;
            myDemes.print_gene_tree(PaupOutFile, tree_type, mut_rate_per_locus);
            write_PAUP_end(PaupOutFile);
            	PaupOutFile
            		<< "\n\t[Tree length = " << (first_node.tree_exp_length()/num_loci)
                  << "\n\tNumber of sites hit by mutations = "  << first_node.count_polym_sites()
                  << "]\n";
				write_PAUP_block(PaupOutFile,mut_mod);
				write_PAUP_footer(PaupOutFile);  
      	#endif

         #ifdef _PRINT_ARLEQUIN_OUTPUT_
            //Build Arlequin file name
            my_string ArlOut;
  				int deca, signa, ndiga = 0;
            ArlOut = paup_generic_name + "_" + fcvt(tot_cases, ndiga, &deca, &signa) + ".arp";

            //Open Arlequin output file
      		ofstream ArlInFile(ArlOut.c_str());


            //Write name in Arlequin batch file
            ArlBatch << ArlOut << "\n";

            ArlInFile
            	<< "#Arlequin input file written by program the simulation program sim_coal.exe\n\n"
      			<< "#Simulation parameters:\n"
               << "#======================\n"
               << "\n#Deme sizes\n";

            for (int i=0; i<num_pop; ++i) {
   				ArlInFile << "#Deme #" << i << "\t" << myDemeSizes[i] << "\n";
   			}
            ArlInFile << "\n#Sample sizes\n";
   			for (int i=0; i<num_pop; ++i) {
   				ArlInFile << "#Deme #" << i << "\t" << mySampleSizes[i] << "\n";
   			}
            ArlInFile << "\n#Growth rates\n";
   			for (int i=0; i<num_pop; ++i) {
   				ArlInFile << "#Deme #" << i << "\t" << myGrowthRates[i] << "\n";
   			}
            for (int i=0; i<numMigMat; ++i) {
            	ArlInFile
            		<< "\n#Migration matrix " << i << "\n"
   					<< myMigrationMatrices[i];
            }
            ArlInFile << "\n\n#Historical events\n";
   			for (int i=0; i<num_events; ++i) {
   				ArlInFile << "#Event #" << i << "\n" << myEvents[i] << "\n";
   			}            
      		ArlInFile
            	<< "\n#Data type							 : ";
            switch (data_type) {
            	case DNA     	: ArlInFile << "DNA (Transition rate="
               									<< setprecision(3)
                                          << (transition_rate*100.0)
                                          << "%)\n";
               					  break;
               case RFLP      : ArlInFile << "RFLP\n"; break;
               case MICROSAT  : {
               						if (range_constraint)
               							ArlInFile
         										<< "MICROSAT"
         										<< " (Range constraint : " << range_constraint
         										<< " [" << (10000+first_node.min_mic) << ";"
                           				<< (10000+first_node.max_mic) << "])\n";
                                 else       
               							ArlInFile
         										<< "MICROSAT"
         										<< " (No range constraint)\n";

                              	}; break;
            }
            ArlInFile
//            	<< setprecision(10)
            	<< "\n#Mutation rate per generation : " << mut_rate
               << "\n#Number of loci to simulate   : " << num_loci
             	<< "\n#Gamma parameter              : " << gamma_a
//            	<< "\n#Mutation ratio               : " << mut_ratio
               << "\n\n";

            //Loro_20_9_99: Get the number of demes with non-empty samples
            int num_real_samples=0;
            for (int d=0; d<num_pop; ++d) {
            	if (myDemes[d].sample_size()) ++num_real_samples;
            }

            write_Arlequin_header(num_real_samples, data_type, ArlInFile);
				myDemes.write_samples_to_Arlequin_file(ArlInFile, data_type);

            myDemes.write_group_section_to_Arlequin_file(ArlInFile);

            ArlInFile.close();

        	#endif

        	#ifdef _COMPUTE_MOMENTS_
         	//Compute the first two moments of coalescence times and mean number
				//of mutation differences over the whole tree
				myDemes.compute_moments_of_coalescence_times();

         	//Compute mean coalescence times and mean number of mutation
            //differences both within and among demes
         	myDemes.compute_moments_of_demes_coalescence_times();

          	MeanCoalMat.update_with(myDemes.coal_time_mat());
         	//Adds up the square of the coalescent times
         	sdCoalMat.update_with_square(myDemes.coal_time_mat());
            MeanPairDiff.update_with(myDemes.mean_pair_diff_mat());
            sdPairDiff.update_with_square(myDemes.mean_pair_diff_mat());
			#endif

      	}
      	myDemes.reset(myDemeSizes,myGrowthRates);
   	}

      if (num_loci) {
         tree_exp_length/=num_loci;
         tree_exp_length2/=num_loci*num_loci;
      }
      else {
         tree_exp_length=0;
         tree_exp_length2=0;
      }

   #ifdef _PRINT_SEQ_PAUP_
		write_PAUP_footer(PaupFile);   
  	#endif
   
   #ifdef _PRINT_TREE_PAUP_
      write_PAUP_end(PaupTrueTreesFile);
      write_PAUP_end(PaupMutTreesFile);
   	PaupTrueTreesFile
      	<< "\nMean length of the trees : "
         << (tree_exp_length/num_rand_samples);
      if (num_rand_samples>1)
      	PaupTrueTreesFile
            << " +- "
         	<< setiosflags(ios::showpoint | ios::fixed | ios::right)
         	<< setw(10)
         	<< setprecision(5)
            << ( sqrt((tree_exp_length2 - tree_exp_length*tree_exp_length/num_rand_samples)/
            			 (num_rand_samples-1)) )
            << endl;

      PaupMutTreesFile
      	<< "\nMean length of the trees : "
         << (tree_mut_length/num_rand_samples);
      if (num_rand_samples>1)
      	PaupMutTreesFile
            << " +- "
         	<< setiosflags(ios::showpoint | ios::fixed | ios::right)
         	<< setw(10)
         	<< setprecision(5)
            << ( sqrt((tree_mut_length2 - tree_mut_length*tree_mut_length/num_rand_samples)/
            			 (num_rand_samples-1)) )
            << endl;
   #endif
        
   	start=clock()-start;
   	cout << "\n\ndone in " <<  start << "ms\n" << endl;

	
   #ifdef _COMPUTE_MOMENTS_
   	MeanCoalMat.compute_mean(num_rand_samples);
   	sdCoalMat.compute_sd(num_rand_samples,MeanCoalMat);

   	MeanPairDiff.compute_mean(num_rand_samples);
   	sdPairDiff.compute_sd(num_rand_samples,MeanPairDiff);
   #endif

   	//Indices for computing pairwise Fst's (Weir's theta)
   	double 	theta, t1, t0, t0_1, t0_2, time;

   	//Print to output file

   	ofs   << "\nFinal results"
   			<< "\n=============\n";

      ofs 	<< "\nData type : ";
      switch (data_type) {
      	case DNA 		: ofs << "DNA"; break;
      	case RFLP 		: ofs << "RFLP"; break;
      	case MICROSAT 	: ofs << "MICROSAT"; break;
      }

      if (data_type==DNA) {
      	if (gamma_a>0.0) {
         	ofs
         		<< "\nHeterogeneity of mutation rates: Gamma value = "
            	<< setiosflags(ios::showpoint | ios::fixed | ios::right)
         		<< setw(10)
         		<< setprecision(5)
         		<< gamma_a;
         }
         else  {
            ofs
         		<< "\nNo heterogeneity of mutation rates";
         }
         ofs
            	<< "\nProportion of transitions : "
            	<< setiosflags(ios::showpoint | ios::fixed | ios::right)
         		<< setw(10)
         		<< setprecision(5) << (100*transition_rate) << " (ts/tv=";
         if (transition_rate!=1)
      		ofs << (transition_rate/(1-transition_rate)) << ")";
      	else
				ofs << "inf";
      }
            
      ofs   << "\nNumber of loci : " << num_loci;

      ofs	<< "\nTotal mutation rate per generation : "
         	<< mut_rate;

      ofs	<< "\nMutation rate per locus per generation : "
         	<< mut_rate/num_loci;
	
   #ifdef _COMPUTE_MOMENTS_
   	ofs 	<< "\nMean coalescence times within and among demes over " << tot_cases <<" random samples\n"
   			<< MeanCoalMat;
   	ofs 	<< "\nS.D. coalescence times within and among demes \n"
   			<< sdCoalMat;
   	ofs 	<< "\nMean number of pairwise differences within and among demes over " << tot_cases <<" random samples\n"
   			<< MeanPairDiff;
   	ofs 	<< "\nS.D. number of pairwise differences within and among demes \n"
   			<< sdPairDiff;
   #endif
   
      ofs
      	<< "\nMean length of the trees (subst. rate per site) : " 
         	<< setiosflags(ios::showpoint | ios::fixed | ios::right)
         << (tree_exp_length/num_rand_samples);
      if (num_rand_samples>1)
      	ofs
            << " +- "
         	<< setiosflags(ios::showpoint | ios::fixed | ios::right)
         	<< setw(10)
         	<< setprecision(5)
            << ( sqrt((tree_exp_length2 - tree_exp_length*tree_exp_length/num_rand_samples)/
            			 (num_rand_samples-1)) )
            << endl;

      ofs
      	<< "\nMean length of the trees (total number of mutations) : "
         << (tree_mut_length/num_rand_samples);
      if (num_rand_samples>1)
      	ofs
            << " +- "
         	<< setiosflags(ios::showpoint | ios::fixed | ios::right)
         	<< setw(10)
         	<< setprecision(5)
            << ( sqrt((tree_mut_length2 - tree_mut_length*tree_mut_length/num_rand_samples)/
            			 (num_rand_samples-1)) )
            << endl;
/*
   #ifdef _COMPUTE_MOMENTS_
      if (num_pop==2) {
      	t0_1=MeanCoalMat(0,0);
      	t0_2=MeanCoalMat(1,1);
			t0=0.5*(t0_1+t0_2);
         t1=MeanCoalMat(1,0);
         if (t1!=0.0)
         	theta=1.0*(t1-t0)/t1;
         else theta=0;
         time=t1-t0;
   		ofs
      		<< "\nt0    = "
   			<< setiosflags(ios::showpoint | ios::fixed | ios::right)
         	<< setw(10)
         	<< setprecision(3)
         	<< t0
   			<< "\nt1    = "
         	<< setiosflags(ios::showpoint | ios::fixed | ios::right)
         	<< setw(10)
         	<< setprecision(3)
         	<< t1
         	<< "\ntheta = "
         	<< setiosflags(ios::showpoint | ios::fixed | ios::right)
         	<< setw(10)
         	<< setprecision(5)
         	<< theta
         	<< "\nDivergence time = "
         	<< setiosflags(ios::showpoint | ios::fixed | ios::right)
         	<< setw(10)
         	<< setprecision(5)
         	<< time;
      }
   #endif
*/      
      ofs
            << "\n\nTotal time for " << tot_cases
         	<< " simulations: " << start << " ms"
         	<< endl;

      //Do not forget to reset the static member of TNode
      TNode resetNode;
      resetNode.reset_node_count();
	}
   return 1;  
};