#include "public.h"
#include "genealgy.h"
#include "deme.h"


enum UNITS {WIDTH=6};

//Initialization of static members of TNode
TIntVect 	TNode::hits(0,10);
GammaRates 	TNode::mut_rates(0,0.0,1.0,0.0, 0);
double 		TNode::mut_rate=0.0;
double 		TNode::tree_length=0.0;
int 			TNode::node_count=0;
int 			TNode::min_mic=0;
int 			TNode::max_mic=0; 

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
TNode&
TNode::operator=(const TNode& N) {
	time=N.time;
   desc1=N.desc1;
   desc2=N.desc2;
   ancestor=N.ancestor;
   num_desc1=N.num_desc1;
   num_desc2=N.num_desc2;
   node_number=N.node_number;
   deme=N.deme;
   num_new_mut=N.num_new_mut;
   left_mut=N.left_mut;
   right_mut=N.right_mut;
   seq_length=N.seq_length;
   if (desc1_nodes) 	delete desc1_nodes; 	desc1_nodes=NULL;
   if (desc2_nodes) 	delete desc2_nodes; 	desc2_nodes=NULL;
   if (sequence) 		delete sequence; 		sequence=NULL;
   if (N.desc1_nodes) {
   	try {
      	int size=N.desc1_nodes->GetItemsInContainer();
      	desc1_nodes= new TIntVect(size,10);
         for (int i=0; i<size; ++i) {
         	desc1_nodes[i]=N.desc1_nodes[i];
         }
      }
      catch (...) {
   		if (desc1_nodes) 	delete desc1_nodes; 	desc1_nodes=NULL;
         cout << "\nTNode::operator=(): unable to allocate memory\n";
         return *this;
      }
   }
   
   if (N.desc2_nodes) {
   	try {
      	int size=N.desc2_nodes->GetItemsInContainer();
      	desc2_nodes= new TIntVect(size,10);
         for (int i=0; i<size; ++i) {
         	desc2_nodes[i]=N.desc2_nodes[i];
         }
      }
      catch (...) {
   		if (desc2_nodes) 	delete desc2_nodes; 	desc2_nodes=NULL;
         cout << "\nTNode::operator=(): unable to allocate memory\n";
         return *this;
      }
   }

   if (N.sequence) {
   	try {
      	int size=N.sequence->GetItemsInContainer();
      	sequence= new TIntVect(size,10);
         for (int i=0; i<size; ++i) {
         	sequence[i]=N.sequence[i];
         }
      }
      catch (...) {
   		if (sequence) 	delete sequence; 	sequence=NULL;
         cout << "\nTNode::operator=(): unable to allocate memory\n";
         return *this;
      }
   }
   return *this;
}
//----------------------------------------------------------------------------
// A recursive method to count the number of nodes below a given node,
// including himself
int
TNode::count_desc() {
	num_desc1=num_desc2=0;
	if (desc1) num_desc1=desc1->count_desc();
	if (desc2) num_desc2=desc2->count_desc();
   //Counts only the basal tips
   if (!desc1 && !desc2) return num_desc1+num_desc2+1;
   return num_desc1+num_desc2;
};
//----------------------------------------------------------------------------
int
TNode::count_polym_sites() {
	int polym_loci=0, size=hits.GetItemsInContainer();
	for (int i=0; i<size; ++i) {
   	if (hits[i]) ++polym_loci;
   }
   return polym_loci;
};
//----------------------------------------------------------------------------
//A recursion procedure to establish the list of descendent nodes to the left
//and to the right of the current node
int
TNode::build_lists_of_descendent_nodes() {

	//Creates empty list of descendent nodes

		try {
   		if (!desc1_nodes) desc1_nodes= new TIntVect(1,10);
         else desc1_nodes->Flush();
      	if (!desc2_nodes) desc2_nodes= new TIntVect(1,10);
         else desc2_nodes->Flush();
   	}
   	catch (...) {
   		if (desc1_nodes) delete desc1_nodes; desc1_nodes=NULL;
      	if (desc2_nodes) delete desc2_nodes; desc2_nodes=NULL;
      	cout 	<< "TNode:: build_list_of_descendent_nodes() : Unable to allocate memory"
      			<< endl;
      	return 0;
   	}

   //Begin recursion
   if (desc1) {
   	//Adds the descendent node list of descent nodes to the current one
   	if (!desc1->build_lists_of_descendent_nodes()) return 0;
      int num_nodes=desc1->desc1_nodes->GetItemsInContainer();
      for (int i=0; i<num_nodes; ++i) {
      	desc1_nodes->Add((*desc1->desc1_nodes)[i]);
      }
      num_nodes=desc1->desc2_nodes->GetItemsInContainer();
      for (int i=0; i<num_nodes; ++i) {
      	desc1_nodes->Add((*desc1->desc2_nodes)[i]);
      }
   }
   else {
   	//Adds its id in its own list of descendent nodes !
   	desc1_nodes->Add(node_number);
   }
   if (desc2) {
   	//Adds the descendent node list of descent nodes to the current one
   	if (!desc2->build_lists_of_descendent_nodes()) return 0;
      int num_nodes=desc2->desc1_nodes->GetItemsInContainer();
      for (int i=0; i<num_nodes; ++i) {
      	desc2_nodes->Add((*desc2->desc1_nodes)[i]);
      }
      num_nodes=desc2->desc2_nodes->GetItemsInContainer();
      for (int i=0; i<num_nodes; ++i) {
      	desc2_nodes->Add((*desc2->desc2_nodes)[i]);
      }
   }
   return 1;
};
//----------------------------------------------------------------------------
//Also a recursion method, assuming that method count_desc()
//has been performed before
int                                                              
TNode::compute_moments_of_pairwise_divergence(double& sum_time, double& sum_square_time) {
  if (desc1 && desc2) {
      //Step 1: The sum of divergence times is simply obtained from the knowledge
      //of the number of descendents
      sum_time+=2.0*time*num_desc1*num_desc2;
      sum_square_time+=4.0*time*time*num_desc1*num_desc2;
      //!!!Beware: This is certainly wrong. No analogy with time
      //Step 2: Now simply recurse through the tree
      desc1->compute_moments_of_pairwise_divergence(sum_time,sum_square_time);
      desc2->compute_moments_of_pairwise_divergence(sum_time,sum_square_time);
   }
   return 1;
};
//----------------------------------------------------------------------------
//A recursion method to compute the total coalescent times,
//both within and among demes
//Could be easily modified to get the distribution of coalescent times
int
TNode::compute_total_coal_times_among_demes( TDemeCollection& DemeCollec,
   													  TMigrationMatrix& CoalTimes,
                                            TMigrationMatrix& PairDiff,
   													  TMigrationMatrix& MinCoalTimes) {
   if (desc1 && desc2) {
   	//Step 1: Count the number of descendent nodes
   	int 	left_desc=desc1_nodes->GetItemsInContainer(),
      		right_desc=desc2_nodes->GetItemsInContainer();

      int deme1, deme2;
      //Step 2: Explore the list of descendent nodes to update coalescence times
      for (int i=0; i<left_desc; ++i) {
      	deme1=DemeCollec.tree_node((*desc1_nodes)[i]-1).deme;
         //Count the descendent nodes belonging to the same deme as deme1
         for (int j=0; j<right_desc; ++j) {
      		deme2=DemeCollec.tree_node((*desc2_nodes)[j]-1).deme;
            if (deme2<deme1)	{  //Between pop treatment
            	CoalTimes(deme1,deme2)+=time;
            	if (time<MinCoalTimes(deme1,deme2)) MinCoalTimes(deme1,deme2)=time;
            }
            else {
            	CoalTimes(deme2,deme1)+=time;
               if (time<MinCoalTimes(deme2,deme1)) MinCoalTimes(deme2,deme1)=time;
            }
         }
      }

      //Step 3: Now count the number of pairwise differences
      int 	size=DemeCollec.num_demes();
      int 	*deme_count_left,*deme_count_right;
      
      try {
      	deme_count_left=new int[size],
      	deme_count_right=new int[size];
         for (int i=0; i<size; ++i) {
         	deme_count_left[i]=0;
         	deme_count_right[i]=0;
         }
      }
      catch (...) {
      	cout << "TNode::compute_total_coal_times_among_demes: unable to allocate memory\n";
         if (deme_count_left) delete[] deme_count_left;
         if (deme_count_right) delete[] deme_count_right;
         return 0;
      }
      //Count the number of nodes of each deme below this node
      for (int i=0; i<left_desc; ++i)
      	++deme_count_left[DemeCollec.tree_node((*desc1_nodes)[i]-1).deme];
      
      for (int i=0; i<right_desc; ++i)
      	++deme_count_right[DemeCollec.tree_node((*desc2_nodes)[i]-1).deme];

      int descnodesi, descnodesj, samp_sizei;
      for (int i=0; i<size; ++i) { 
      	descnodesi=deme_count_left[i]+deme_count_right[i];
         samp_sizei=DemeCollec[i].sample_size();
      	//Between pop treatment
      	for (int j=0; j<i; ++j) {
         	descnodesj=deme_count_left[j]+deme_count_right[j];
            PairDiff(i,j)+=num_new_mut*
            					( descnodesi*(DemeCollec[j].sample_size()-descnodesj)+
                            (descnodesj)*(samp_sizei-descnodesi) );
         }
         //Within pop treatment
         PairDiff(i,i)+=num_new_mut*descnodesi*(samp_sizei-descnodesi); //OK
      }
      //Recurse through the tree
      desc1->compute_total_coal_times_among_demes(DemeCollec,CoalTimes, PairDiff, MinCoalTimes);
      desc2->compute_total_coal_times_among_demes(DemeCollec,CoalTimes, PairDiff, MinCoalTimes);

      if (deme_count_left) delete[] deme_count_left;
      if (deme_count_right) delete[] deme_count_right;

   }
   else {
   	PairDiff(deme,deme)+=num_new_mut*(DemeCollec[deme].sample_size()-1);
      for (int i=0; i<DemeCollec.num_demes(); ++i) {
      	if (deme>i)
         	PairDiff(deme,i)+=num_new_mut*DemeCollec[i].sample_size();
         else if (i>deme)
         	PairDiff(i,deme)+=num_new_mut*DemeCollec[i].sample_size();
      }
   }
	return 1;
};
//----------------------------------------------------------------------------
ostream & operator<<(ostream& os, const TNode& node) {
	if (node.time>0) {
		os << "\nInternal node No " << node.node_number
      	<< "\n----------------"
      	<< "\nTime since present       : " << node.time
   		<< "\nNo. of left descendents  : " << node.num_desc1
   		<< "\nNo. of right descendents : " << node.num_desc2
/*      	<< "\nLeft descendent address  : " << node.desc1
      	<< "\nRight descendent address : " << node.desc2
      	<< "\nAncestor address         : " << node.ancestor */<< endl;
	}
   else os << "\nTip node No " << node.node_number
           << "\n-----------" << endl;
   return os;
};
//----------------------------------------------------------------------------
// A recursive method to print node information recursively
int
TNode::print_info(ostream& os) {
	os << *this;
	if (desc1) desc1->print_info(os);
	if (desc2) desc2->print_info(os);
   return 1;
};
/*
void
TNode::print_desc_nodes(TDrawingBoard& DB, int node_posx, int node_posy,
      								char node, char hor_bar, char left_corner,
                              char right_corner, char vert_bar) {
   //Draw Node number
   char num[20];
   itoa(this->node_number, num, 10);
   DB.draw_text(node_posx, node_posy+1,num);
   if (desc1 && desc2) {
   	int l11, l12, r11, r12;
      l11=node_posx-(int)((float)(num_desc1+num_desc2-1)*WIDTH)/2;
      l12=l11+(num_desc1-1)*WIDTH;
      r12=l11+(num_desc1+num_desc2-1)*WIDTH;
      r11=r12-(num_desc2-1)*WIDTH;
   	int pos_vert_line1=(int)((float)(l11+l12)/2),
          pos_vert_line2=(int)((float)(r11+r12)/2),
          vert_line1_height=(int)((this->time-desc1->time)*UNIT_TIME)-2,
          vert_line2_height=(int)((this->time-desc2->time)*UNIT_TIME)-2;
   	//Draw left corner
   	DB.draw_char(pos_vert_line1, node_posy, left_corner);
		//Draw first line segment                         mod
      DB.draw_hor_line(pos_vert_line1+1,node_posy, node_posx-pos_vert_line1-1, hor_bar);
   	//Draw node
   	DB.draw_char(node_posx, node_posy, node);
		//Draw second line
   	DB.draw_hor_line(node_posx+1,node_posy, pos_vert_line2-node_posx-1, hor_bar);
   	//Draw right corner
      DB.draw_char(pos_vert_line2, node_posy, right_corner);
      //Draw vertical lines until next nodes
      DB.draw_vert_line(pos_vert_line1,node_posy+1, vert_line1_height, vert_bar);
      DB.draw_vert_line(pos_vert_line2,node_posy+1, vert_line2_height, vert_bar);
		//Call descendents methods
      desc1->print_desc_nodes(DB,pos_vert_line1,node_posy+vert_line1_height+2,
      								node,hor_bar,left_corner, right_corner, vert_bar);
      desc2->print_desc_nodes(DB,pos_vert_line2,node_posy+vert_line2_height+2,
      								node,hor_bar,left_corner, right_corner, vert_bar);
   }
};
*/
//----------------------------------------------------------------------------
//A recursive procedure to add mutations to the tree starting from the root
long
TNode::add_mutations(long *lidum, const int& num_mut,
      		const int& len, const Mut_Type& mut_type, const float& gamma_par,
            const float& trans_rate, const int& range_const) {

   long tot_mut=0L, desc_mut;

   float subst_length;

   seq_length=len;

   //Create new sequence and propagate the ancestor's mutations
   if (!sequence)  {
   	try {
    		sequence= new TIntVect(len, 0);
   	}
   	catch (...) {
   		if (sequence) delete sequence; sequence=NULL;
      	seq_length=0;
      	return 0;
   	}
   }
   if (ancestor) {
   	int *cursite=sequence->begin(), *cursite_anc=ancestor->sequence->begin();
   	for (int i=0; i<len; ++i) {
      	*cursite++=*cursite_anc++; //Inheritance of ancestor's mutations
      	//(*sequence)[i]=(*ancestor->sequence)[i]; //Inheritance of ancestor's mutations
      }
   }
   else  {    //Loro_15_9_98 Create the ancestor's genetic data
   	int* cursite=sequence->begin();
   	if (mut_type==DNA) {
      	 //for (int i=0; i<len; ++i) (*sequence)[i]=ran3(lidum)*4; //A random number between 0 and 3
			for (int i=0; i<len; ++i) *cursite++=(int)(ran3(lidum)*4); //A random number between 0 and 3
      }
      //else for (int i=0; i<len; ++i) (*sequence)[i]=0;
      else for (int i=0; i<len; ++i) *cursite++=0;

   }

   //This is the number of new mutations to generate as compared to parent node
   num_new_mut=num_mut;

   double drand_num;
   //Generate those mutations
   for (int i=0, pos; i<num_mut; ++i) {
      //Find the position of the site to be hit by a mutation
   	if (fabs(gamma_par)<1e-7) { //Close to zero, even mutation rates are assumed among sites
      	pos=(int) (ran3(lidum)*len);
      }
      else {
      	drand_num=ran3(lidum);
         double tot_prob=0.0;
         /*
         for (pos=-1; pos<(len-1); ) {
            ++pos;
         	tot_prob+=mut_rates[pos];
            if (drand_num<tot_prob) break;
         }
         */
         //Explore from the end of the sequence because it is where mutatins are
         //more likely to occur
         for (pos=len-1; pos>-1; --pos) {
         	tot_prob+=mut_rates[pos];
            if (drand_num<tot_prob) break;
         }
      }
      ++hits[pos];
   	int& site=(*sequence)[pos];
   	switch (mut_type) {
      	case MICROSAT  :	//Loro_26_2_99
         					  	if (range_const) {
                           	//There are bouncing walls at min_mic and max_mic
                           	if (site==min_mic) ++site;
                              else if (site==max_mic) --site;
                              //otherwise they are free to move randomly
                              else if (ran3(lidum)<0.5) --site;
         							else ++site;
         						}
                           else {
         							if (ran3(lidum)<0.5) --site;
         							else ++site;
                           }
         				 		break;
         case DNA			:  drand_num=ran3(lidum);
         						//Here we implement a 95% transition bias
                           //0 and 1 : A and G
                           //2 and 3 : C and T 
                           switch (site) {
                           case 0: 	if (drand_num<trans_rate) site=1;
                           			else
                                    	if (ran3(lidum)<0.5) site=2;
                                       else site=3;
                                    break;
                           case 1: if (drand_num<trans_rate) site=0;
                           			else
                                    	if (ran3(lidum)<0.5) site=2;
                                       else site=3;
                                    break;
                           case 2: if (drand_num<trans_rate) site=3;
                           			else
                                    	if (ran3(lidum)<0.5) site=0;
                                       else site=1;
                                    break;
                           case 3: if (drand_num<trans_rate) site=2;
                           			else
                                    	if (ran3(lidum)<0.5) site=0;
                                       else site=1;
                                    break;
                           }
         						break;
         case RFLP		: 	site=!site;
         						break;
         default /*DNA*/: break;
      }
   }

   //Continue recursion
   if (desc1) {
   	subst_length=mut_rate*(time-desc1->time);
      desc_mut= (int) poidev(subst_length, lidum);
      tree_length+=subst_length;
      tot_mut+=desc_mut+desc1->add_mutations(lidum, desc_mut, len, mut_type,
      													gamma_par,trans_rate, range_const);
   }
   if (desc2) {
   	subst_length=mut_rate*(time-desc2->time);
      desc_mut= (int) poidev(subst_length, lidum);
      tree_length+=subst_length;
      tot_mut+=desc_mut+desc2->add_mutations(lidum, desc_mut, len, mut_type,
      													gamma_par,trans_rate, range_const);
   }
   return tot_mut;
};
//----------------------------------------------------------------------------
//A recursive routine to print tree topology and branch length
//Loro 27.8.98
void
TNode::print_tree_structure(ostream& os, const tree_output_type tree_type, const float & mu) {
	if (desc1 && desc2) os << "(";
   if (desc1) desc1->print_tree_structure(os, tree_type, mu);
   if (desc1 && desc2) {
   	os << ", ";
   }
   else {
   	os << node_number << "." << (deme+1);
   }
   if (desc2) desc2->print_tree_structure(os, tree_type, mu);
   if (desc1 && desc2) os << ")";
   if (ancestor) {
   	switch (tree_type)  {
      	case GENERATIONS : 	os << ":" << (ancestor->time-time);
                              break;
      	case MUT_RATE    : 	os << ":" << (ancestor->time-time)*mu;
                              break;
      	case NUM_MUT     : 	os << ":" << num_new_mut;
                              break;
      }
   }
   else os << ";\n";
};
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
int
TListNode::allocate_list(const int size) {

	if (list) delete[] list;

   try {
		list = new TNode*[size];
      _size=size;
   }
   catch (...) {
   	if (list) delete[] list;
      list=NULL;
      _size=0;
      cout << "TListNode::allocate_list : unable to allocate memory" << endl;
      return 0;
   }
   return 1;
}
//----------------------------------------------------------------------------
TListNode::TListNode(const int size, TNode * tree) {
	if (tree)
	try {
		list = new TNode*[size];
      for (int i=0; i<size; ++i) {
      	tree[i].time=0;
      	tree[i].desc1=tree[i].desc2=NULL;
         list[i]=tree + i;
      }
      ListSize=size;
   }
   catch (...) {
   	if (list) delete[] list;
      list=NULL;
      _size=0;
      cout << "TListNode::TListNode(const int size, TNode * tree) : unable to allocate memory" << endl;
      return;
   }
};
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

TTree::TTree(int size) {
	SampleSize=size;
   delete_nodes=1;
   _mean_coalescence_time=0;
   _sd_coalescence_time=0;
   try {
   	tree=new TNode[2*SampleSize];
   }
   catch (...) {
   	if (tree) delete[] tree; tree=NULL;
      cout << "TTree::TTree(int size) : unable to allocate memory" << endl; 
   }
};
//----------------------------------------------------------------------------
int
TTree::allocate_tree(const int& sampsize) {                           
	SampleSize=sampsize;
	int array_size=2*SampleSize-1;
   //Create tree structure if needed
   if (tree) delete[] tree;
   try {
   	tree=new TNode[array_size];
   }
   catch (...) {
   	if (tree) delete[] tree;
      return 0;
   }
   return 1;
};
//----------------------------------------------------------------------------
//Hudson original method
int
TTree::build_tree() {
       
 	double t;

   //Node list initialisation
   TListNode List(SampleSize, tree);

   long lidum=1L;

   //Generate the times of the nodes
   t=0.0;
   for (int i=SampleSize; i>1 ; --i) {
   	t+=-2.0*log(1.0-ran3(&lidum)) / ( (double) i*(i-1));
      tree[2*SampleSize-i].time= (long) t;
      cout <<	(2*SampleSize-i) << "\t" << t << endl;
   }
   //Generate the topology of the tree
   for (int i=SampleSize, pick, pos; i>1 ; --i) {
   	pick= (int) (ran3(&lidum)*i);
      pos=2*SampleSize-i;
      List[pick]->ancestor=tree+pos;
      tree[pos].desc1=List[pick];
      List[pick]=List[i-1];
      pick= (int) ((i-1)*ran3(&lidum));
      List[pick]->ancestor=tree+pos;
      tree[pos].desc2=List[pick];
      List[pick]=tree+pos;
   }
   return 1;
};
//----------------------------------------------------------------------------
int
TTree::bottleneck(double time, double factor) {
	for (int i=SampleSize; i<2*SampleSize-1; ++i) {
   	if (tree[i].time > time) {
      	cout << "\nTime before bottleneck : " << tree[i].time;
      	tree[i].time = (long) (factor*(tree[i].time - time) + time);
      	cout << "\nTime after bottleneck  : " << tree[i].time << "\n";
      }
   }
   return 1;
};
//----------------------------------------------------------------------------
//A routine to compute the first two moments of coalescence times and mean number
//of mutation differences over the whole tree
int
TTree::compute_moments_of_coalescence_times() {

   _mean_coalescence_time=0.0;
   _sd_coalescence_time=0.0;

	double 	sumtime=0.0, sumsquaretime=0.0,
   			num_pairwise=SampleSize*(SampleSize-1)*0.5;
   if (!num_pairwise) return 0;

	//Root the tree
   TNode& root=tree[2*SampleSize-2];
   root.count_desc();  
   root.compute_moments_of_pairwise_divergence(sumtime, sumsquaretime);

   _mean_coalescence_time=0.5*sumtime/num_pairwise;
   if (num_pairwise>1)  {
   	_sd_coalescence_time=0.5*sqrt( (sumsquaretime-sumtime*sumtime/num_pairwise)/
      										 (num_pairwise-1) );
   }

   return 1;
};
/*
void
TTree::print_nodes(ostream& os, double factor, double time) {

	//Root the tree
	TNode& root=tree[2*SampleSize-2];

   //Create a drawing board
   int 	num_lines=root.time*UNIT_TIME+1,
   		num_col=(SampleSize-1)*WIDTH+2;
   TDrawingBoard DB(num_lines,num_col);


	//Count number of descendents starting from the root of the tree using recursions
//   cout << "\tCounting descendent nodes fron the root\n";
   root.count_desc();
//   cout << "\tDone\n";

   //Printing node information
   os << "\nNode information"
   	<< "\n================\n"
      << "\n Sample size : " << (root.num_desc1+root.num_desc2) << "\n\n";
//   cout << "\tPrinting node information, starting from the root\n";
   root.print_info(os);
//   cout << "\tDone\n";

   //Define how the tree will be drawn
   char node='o', hor_bar='-', left_corner='/', right_corner='\\', vert_bar='|';

   //Get the horizontal position of the root node : Middle of the first horizontal segment
   int node_posx=(int)((float)(root.num_desc1+root.num_desc2-1)*WIDTH/2),
   	 node_posy=0;

   //Call a recursive printing procedure from the root
   root.print_desc_nodes(DB, node_posx, node_posy, node, hor_bar, left_corner,
                          right_corner, vert_bar); 
   os << "\f\nTree structure"
   	<< "\n==============\n"
      << "Sample size : " << (root.num_desc1+root.num_desc2)
      << "\t Expansion factor : " << factor
      << "\t Expansion time : " << time
      << "\n 1 unit of time : " << UNIT_TIME << " characters\n\n";
   os << DB << endl;
};
*/
//----------------------------------------------------------------------------
void
TTree::print_nodes(ostream& os, const tree_output_type tree_type, const float & mu) {
	//Root the tree
	TNode& root=tree[2*SampleSize-2];
   root.print_tree_structure(os, tree_type, mu);
}
//----------------------------------------------------------------------------
void
TTree::print_sequences(ostream& os, const Mut_Type& mut_type) {
   int len=tree[0].seq_length;
   char DNA_letters[4]={'A','G','C','T'};
   if (len) {
   	//Prints first the number of hits per site
   	os << "Hits\t";
		for (int j=0; j<len; ++j) {
      	os << tree[0].hits[j] << " ";
      }
      os << "\n\n";

      //Then prints the sequences
      for (int i=0; i<SampleSize; ++i) {
   		os << (i+1) << "\t\t";
      	if (tree[i].sequence) {
         	int* cursite=tree[i].sequence->begin();
   			for (int j=0; j<len; ++j) {
            	if (mut_type==DNA)
   					//os << DNA_letters[(*tree[i].sequence)[j]];
                  os << DNA_letters[*cursite++];
               else
   					//os << (*tree[i].sequence)[j];
                  //Loro_1_3_99 : In order to avoid negative numbers
               	if (mut_type==MICROSAT) os << (10000+*cursite++) << " ";
               	else os << *cursite++;
            }
      		os << "\n";
      	}
   	}
   }
   os << "\n";
};
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------


